
"""
__version__ = "$Revision: 1.2 $"
__date__ = "$Date: 2002/11/18 06:40:07 $"
"""

from wxPython import wx
import sys

from PythonCardPrototype import binding, event, registry, widget
import textfield
from wxPython.lib.rightalign import wxRightTextCtrl

class RightTextFieldSpec(widget.WidgetSpec):
    def __init__(self):
        widget.WidgetSpec.__init__(self)
        
        self.name = 'RightTextField'
        self.parent = 'Widget'
        self.parentName = self.parent
        self.events.extend([event.KeyPressEvent, 
                event.KeyDownEvent, 
                event.KeyUpEvent, 
                #event.TextEnterEvent,
                event.TextUpdateEvent,
                event.CloseFieldEvent
        ])
        self._attributes.update({
            'text' : { 'presence' : 'optional', 'default' : '' },
            'editable' : { 'presence' : 'optional', 'default' : 1 },
            'border' : { 'presence' : 'optional', 'default' : '3d', 'values' : [ '3d', 'none' ] }
        })
        
        self.attributes = self.parseAttributes(self._attributes)
        self.requiredAttributes = self.parseRequiredAttributes()
        self.optionalAttributes = self.parseOptionalAttributes()


class RightTextField(widget.Widget, wxRightTextCtrl):
    """
    A right-aligned text field.
    """

    _spec = RightTextFieldSpec()

    def __init__( self, aParent,  aResource ) :
        attributes = ['_border']
        self._createAttributes(attributes)
        self._border = aResource.border
        widget.Widget.__init__( self, aParent, aResource )

        # previously _createDelegate would be called by Widget.__init__
        # so put wxTextCtrl.__init__ here
        if aResource.border == 'none':
            borderStyle = wx.wxNO_BORDER
        else:
            borderStyle = 0

        wxRightTextCtrl.__init__(
            self,
            aParent, 
            self.getId(), 
            aResource.text, 
            wx.wxPoint( aResource.position[ 0 ], aResource.position[ 1 ] ), 
            wx.wxSize( aResource.size[ 0 ], aResource.size[ 1 ] ), 
            #style = wxTE_PROCESS_ENTER | borderStyle | wxCLIP_SIBLINGS,
            style = borderStyle | wx.wxCLIP_SIBLINGS,
            name = aResource.name )

        if not hasattr(self.__class__, '_getText'):
            self.__class__._getText = self.__class__.GetValue
        if not hasattr(self.__class__, '_setText'):
            self.__class__._setText = self.__class__.SetValue
        if not hasattr(self.__class__, '_getEditable'):
            self.__class__._getEditable = self.__class__.IsEditable
        if not hasattr(self.__class__, '_setEditable'):
            self.__class__._setEditable = self.__class__.SetEditable

        self._setEditable( aResource.editable )

        if aResource.border == 'none':
            # the erase background event doesn't appear to make the control
            # transparent, so further investigation is required
            #EVT_ERASE_BACKGROUND(delegate, lambda evt: None)
            self.SetBackgroundColour(self.GetParent().GetBackgroundColour())

        # then call Widget._postInit which were the initialization
        # steps done after the _delegate was created
        widget.Widget._postInit(self, aParent, aResource)

    # KEA 2002-11-17
    # this is an elaborate hack to get closeField
    # events working in RightTextField
    # but loseFocus events are still lost because of the
    # duplicate EVT_KILL_FOCUS bindings
    def OnKillFocus(self, aWxEvent):
        if not self.GetParent(): return
        self.Refresh()
        try:
            obj = aWxEvent.GetEventObject()
            if obj.IsModified():
                closeFieldEvent = wx.wxWindowCreateEvent()
                closeFieldEvent.SetEventType(event.wxEVT_CLOSE_FIELD)
                closeFieldEvent.SetEventObject(aWxEvent.GetEventObject())
                closeFieldEvent.SetId(aWxEvent.GetId())
                closeFieldEvent.SetTimestamp(aWxEvent.GetTimestamp())
                # this is what Robin suggested instead, see:
                # http://aspn.activestate.com/ASPN/Mail/Message/wxPython-users/1103427
                #obj.GetParent().GetEventHandler().ProcessEvent(closeFieldEvent)
                obj.GetEventHandler().ProcessEvent(closeFieldEvent)
            else:
                aWxEvent.Skip()
        except:
            pass

    def _bindEvents(self):
        adapter = textfield.TextFieldEventBinding(self)
        adapter.bindEvents()

    # KEA the methods for retrieving and manipulating the text
    # has to be greatly expanded to match wxPython
    # capabilities or more

##    def _setText( self, aString ) :
##        """Sets the text value and marks the widget as not-modified.
##        aString may contain newline characters if the text widget is multi-line."""
##        self.SetValue( aString )
##
##    def _getText( self ) :
##        """Gets the contents of the widget. Notice that for a multiline text
##        widget, the lines will be separated by (Unix-style) \\n characters,
##        even under Windows where they are separated by a \\r\\n sequence in
##        the native control."""
##        return self.GetValue()
##
##    def _getEditable( self ) :
##        """Returns whether the text widget is editable or read-only."""
##        # KEA the test below doesn't seem to be working, so...
##        #return not self._delegate.GetWindowStyleFlag() & wxTE_READONLY
##        return self.IsEditable()
##
##    def _setEditable( self, aBoolean ) :
##        """Makes the text widget editable or read-only."""
##        self.SetEditable( aBoolean )

    # KEA new methods to mirror wxPython wxTextCtrl capabilities
    def appendText( self, aString ) :
        """Appends the text to the end of the text widget.
        After the text is appended, the insertion point will be at the end
        of the text widget. If this behavior is not desired, the programmer
        should use getInsertionPoint and setInsertionPoint."""
        self.AppendText( aString )

    def canCopy( self ) :
        return self.CanCopy()

    def canCut( self ) :
        return self.CanCut()

    def canPaste( self ) :
        return self.CanPaste()

    def canRedo( self ) :
        return self.CanRedo()

    def canUndo( self ) :
        return self.CanUndo()

    def clear( self ) :
        self.Clear()

    def copy( self ) :
        self.Copy()

    def cut( self ) :
        self.Cut()

    def discardEdits( self ) :
        self.DiscardEdits()
    
    def getInsertionPoint( self ) :
        return self.GetInsertionPoint()

    def getLastPosition( self ) :
        return self.GetLastPosition()

    def getLineLength( self, aLineNumber ) :
        return self.GetLineLength( aLineNumber )

    def getLineText( self, aLineNumber ) :
        return self.GetLineText( aLineNumber )

    def getNumberOfLines( self ) :
        return self.GetNumberOfLines()

    def getSelection( self ) :
        return self.GetSelection()

    def getNumberOfLines( self ) :
        return self.GetNumberOfLines()

    # KEA rename to getModified?
    def isModified( self ) :
        """Returns 1 if the text has been modified, 0 otherwise."""
        return self.IsModified()

    # KEA support LoadFile? If so, it only makes sense for TextArea
    # many of the other methods only make sense for the multiline TextArea
    # not TextField and PasswordField

    # KEA OnChar ties into our user code handlers and our events,
    # need to think about this one some more

    # KEA OnDropFiles is windows-specific, if you try and call it under *nix
    # what happens? just an exception?

    def paste( self ) :
        self.Paste()

    def positionToXY(self, aPosition):
        result = self.PositionToXY(aPosition)
        if len(result) == 2:
            return result
        else:
            # workaround for wxPython 2.3.2.1
            return (result[1], result[2])

    def redo( self ) :
        self.Redo()

    def remove( self, aFrom, aTo ) :
        self.Remove( aFrom, aTo )

    def replace( self, aFrom, aTo, aString ) :
        # KEA workaround for Replace bug, has the side effect of
        # possibly changing the insertion point
        #self._delegate.Replace( aFrom, aTo, aString )
        i = self.GetInsertionPoint()
        self.Remove( aFrom, aTo )
        self.SetInsertionPoint( aFrom )
        self.WriteText( aString )
        self.SetInsertionPoint( i )

    def replaceSelection(self, aString, select=0):
        """
        if wxPlatform == "__WXMSW__":
            if select:
                sel = self._delegate.GetSelection()
                numNewlines = aString.count('\n')
                self._delegate.WriteText(aString)
                self._delegate.SetSelection( sel[0], sel[0] + len(aString) + numNewlines)
            else:
                self._delegate.WriteText(aString)
        else:
            # Linux
        """
        sel = self.GetSelection()
        self.Remove(sel[0], sel[1])
        self.WriteText(aString)
        if select:
            self.SetSelection(sel[0], sel[0] + len(aString))
        """
        ins = self._delegate.GetInsertionPoint()
        sel = self._delegate.GetSelection()
        self._delegate.Remove( sel[0], sel[1] )
        #self._delegate.SetInsertionPoint( aFrom )
        self._delegate.WriteText( aString )
        self._delegate.SetSelection( ins, ins + len(aString))
        #self._delegate.SetInsertionPoint( i )
        """

    # KEA support SaveFile?

    def setInsertionPoint( self, aPosition ) :
        self.SetInsertionPoint( aPosition )

    def setInsertionPointEnd( self ) :
        self.SetInsertionPointEnd()

    def setSelection( self, aFrom, aTo ) :
        self.SetSelection( aFrom, aTo )

    def showPosition( self, aPosition ) :
        self.ShowPosition( aPosition )

    def undo( self ) :
        self.Undo()

    def writeText( self, aString ) :
        self.WriteText( aString )

    def xyToPosition( self, aX, aY ) :
        return self.XYToPosition( aX, aY )

    def _getBorder( self ) :
        return self._border

    def _setBorder( self, aString ) :
        raise AttributeError, "border attribute is read-only"

    # KEA added 2001-11-13
    # this will be replaced once 2.3.2 has GetStringSelection
    def getStringSelection(self):
        sel = self.getSelection()
        txt = self._getText()
        """
        if wxPlatform == '__WXMSW__':
            numNewlines = txt.count('\n', 0, sel[0])
            start = sel[0] - numNewlines
            # have to special-case a selection to the end of the line
            # this is in a try block to avoid an exception when we index past the end
            # of the string
            try:
                if txt[start + 1] == "\n":
                    start = start + 1
            except:
                pass
            #print numNewlines, start
            numNewlines = txt.count('\n', 0, sel[1])
            end = sel[1] - numNewlines
            # have to special-case a selection to the end of the line
            # this is in a try block to avoid an exception when we index past the end
            # of the string
            try:
                if txt[end + 1] == "\n":
                    end = end + 1
            except:
                pass
            #print numNewlines, end
            selectedText = txt[start:end]
            #print selectedText
        else:
        """
        selectedText = txt[sel[0]:sel[1]]
        return selectedText

    def getString(self, aFrom, aTo):
        return self.GetValue()[aFrom:aTo]


registry.getRegistry().register(sys.modules[__name__].RightTextField)
